package weaver.formmode.mzg.modeexpand.interview;

import com.engine.mzg.conn.RecruitCommon;
import com.engine.mzg.enums.EmailTemplateEnum;
import com.engine.mzg.exception.CustomizeRunTimeException;
import com.engine.mzg.util.RecruitUtil;
import com.weaver.formmodel.data.model.Formfield;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.formmode.IgnoreCaseHashMap;
import weaver.formmode.customjavacode.AbstractModeExpandJavaCodeNew;
import weaver.general.BaseBean;
import weaver.general.Util;
import weaver.soa.workflow.request.MainTableInfo;
import weaver.soa.workflow.request.Property;
import weaver.soa.workflow.request.RequestInfo;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author:dxfeng
 * @createTime: 2024/09/10
 * @version: 1.0
 */
public class AdjustmentInterviewFlowExpand extends AbstractModeExpandJavaCodeNew {
    @Override
    public Map<String, String> doModeExpand(Map<String, Object> param) {
        // 取消面试
        Map<String, String> result = new HashMap<>();
        try {
            String billId;
            String recruitType = Util.null2String(param.get("recruitType"));
            if (!"adjustmentInterview".equals(recruitType)) {
                return result;
            }
            RequestInfo requestInfo = (RequestInfo) param.get("RequestInfo");
            if (requestInfo != null) {
                billId = requestInfo.getRequestid();
                RecordSet rs = new RecordSet();
                IgnoreCaseHashMap<String, String> mainDataMap = new IgnoreCaseHashMap<>();
                MainTableInfo mainTableInfo = requestInfo.getMainTableInfo();
                Property[] properties = mainTableInfo.getProperty();
                for (Property property : properties) {
                    mainDataMap.put(property.getName(), property.getValue());
                }
                String mslx = Util.null2String(mainDataMap.get("mslx"));
                String mslc = "";
                String msrq = "";
                String msfs = "";
                String msdz = "";
                Integer emailTemplateId;
                if ("0".equals(mslx)) {
                    mslc = "dyl";
                    emailTemplateId = EmailTemplateEnum.INTERVIEW_FIRST.getValue();
                } else if ("1".equals(mslx)) {
                    mslc = "del";
                    emailTemplateId = EmailTemplateEnum.INTERVIEW_SECOND.getValue();
                } else if ("2".equals(mslx)) {
                    mslc = "dsl";
                    emailTemplateId = EmailTemplateEnum.INTERVIEW_THIRD.getValue();
                } else {
                    result.put("errmsg", "面试类型匹配异常");
                    result.put("flag", "false");
                    return result;
                }
                msrq = Util.null2String(mainDataMap.get(mslc + "msrq"));
                msfs = Util.null2String(mainDataMap.get(mslc + "msfs"));
                msdz = Util.null2String(mainDataMap.get(mslc + "msdz"));

                // 更新流程中的字段
                String flowTableInterview = RecruitCommon.getSettingValue("FLOW_TABLE_INTERVIEW");
                String requestId = Util.null2String(mainDataMap.get("mslcid"));
                String sql = "update " + flowTableInterview + " set " + mslc + "msrq  = ?," + mslc + "msfs=?," + mslc + "msdz=? where requestId = ? ";
                rs.executeUpdate(sql, msrq, msfs, msdz, requestId);

                // 发送调整面试邮件
                String emailTitle = "";
                String emailContent = "";
                rs.executeQuery("select yjzt ,yjnr  from uf_recruit_email where mblx =? ", emailTemplateId);
                if (rs.next()) {
                    emailTitle = rs.getString("yjzt");
                    emailContent = rs.getString("yjnr");
                }

                if (StringUtils.isBlank(emailTitle) || StringUtils.isBlank(emailContent)) {
                    throw new CustomizeRunTimeException("请检查邮件模板设置");
                }
                List<Formfield> fieldList = RecruitUtil.getFieldList("uf_recruit_ms");
                Map<String, List<Formfield>> fieldMapList = fieldList.stream().collect(Collectors.groupingBy(Formfield::getFieldname));
                emailContent = RecruitUtil.getReplaceContent(emailContent, fieldMapList, mainDataMap);
                String sendTo = Util.null2String(mainDataMap.get("dzyx"));
                RecruitUtil.sendEmail(sendTo, emailTitle, emailContent);
            }
        } catch (Exception e) {
            new BaseBean().writeLog(e);
            result.put("errmsg", e.getMessage());
            result.put("flag", "false");
        }
        return result;
    }
}
