package weaver.interfaces.mzg.action.interview;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.engine.common.util.ServiceUtil;
import com.engine.mzg.conn.RecruitCommon;
import com.engine.mzg.enums.EmailTemplateEnum;
import com.engine.mzg.util.RecruitUtil;
import com.engine.workflowDesign.biz.AutoForecastBiz;
import com.engine.workplan.service.impl.WorkPlanBaseServiceImpl;
import com.weaver.formmodel.data.manager.FormInfoManager;
import com.weaver.formmodel.data.model.Formfield;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.formmode.IgnoreCaseHashMap;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.MainTableInfo;
import weaver.soa.workflow.request.Property;
import weaver.soa.workflow.request.RequestInfo;
import weaver.workflow.request.RequestNodeFlow;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author:dxfeng
 * @createTime: 2024/09/11
 * @version: 1.0
 */
public class SubmitInterviewAction implements Action {
    private String interviewStep;

    @Override
    public String execute(RequestInfo requestInfo) {

        RecordSet rs = new RecordSet();
        try {
            if (StringUtils.isBlank(interviewStep)) {
                requestInfo.getRequestManager().setMessagecontent("面试流程配置异常，请检查");
                return FAILURE_AND_CONTINUE;
            }
            MainTableInfo mainTableInfo = requestInfo.getMainTableInfo();
            int billId = requestInfo.getRequestManager().getBillid();
            String billTableName = requestInfo.getRequestManager().getBillTableName();
            int formId = requestInfo.getRequestManager().getFormid();
            User user = requestInfo.getRequestManager().getUser();
            int requestId = requestInfo.getRequestManager().getRequestid();
            int nodeId = requestInfo.getRequestManager().getNodeid();
            rs.executeQuery("select mszt from " + billTableName + " where id = ?", billId);
            if (rs.next()) {
                String mszt = rs.getString("mszt");
                if ("2".equals(mszt)) {
                    requestInfo.getRequestManager().setMessagecontent("该面试已取消");
                    return FAILURE_AND_CONTINUE;
                }
            }


            Set<String> userIdSet = new HashSet<>();

            RequestNodeFlow flow = new AutoForecastBiz().getRequestNodeFlow(requestId, Util.getIntValue(nodeId), user);
            flow.getNextNodes();
            ArrayList operatorshts = flow.getOperatorshts();
            if (CollectionUtils.isNotEmpty(operatorshts)) {
                for (Object obj : operatorshts) {
                    JSONObject jsonObject = (JSONObject) JSON.toJSON(obj);
                    // 使用流式处理遍历 JSONObject 的所有键
                    jsonObject.keySet().stream().forEach(key -> {
                        JSONArray jsonArrayForKey = jsonObject.getJSONArray((String) key);
                        if (jsonArrayForKey.size() > 0) {
                            String stringValue = jsonArrayForKey.getString(0);
                            // 分割字符串并提取数字
                            String[] parts = stringValue.split("_");
                            if (parts.length > 0) {
                                userIdSet.add(parts[0]);
                            }
                        }
                    });
                }

            }

            // 去除创建者
            userIdSet.remove(String.valueOf(user.getUID()));

            IgnoreCaseHashMap<String, String> mainDataMap = new IgnoreCaseHashMap<>();
            Property[] propertyArray = mainTableInfo.getProperty();
            for (Property property : propertyArray) {
                mainDataMap.put(property.getName(), property.getValue());
            }

            Optional<String> userIdOptional = userIdSet.stream().findFirst();
            String msg = null;
            if (userIdOptional.isPresent()) {
                msg = userIdOptional.get();
            }

            // 根据轮次
            String msgKey;
            String msgField;
            String msrqField;
            Integer emailTemplate;
            if ("0".equals(interviewStep)) {
                msgKey = "MSG_INTERVIEEW_FIRST";
                msgField = "dylmsg";
                msrqField = "dylmsrq";
                emailTemplate = EmailTemplateEnum.INTERVIEW_FIRST.getValue();
            } else if ("1".equals(interviewStep)) {
                msgKey = "MSG_INTERVIEW_SECOND";
                msgField = "delmsg";
                msrqField = "delmsrq";
                emailTemplate = EmailTemplateEnum.INTERVIEW_SECOND.getValue();
            } else if ("2".equals(interviewStep)) {
                msgKey = "MSG_INTERVIEW_THIRD";
                msgField = "dslmsg";
                msrqField = "dslmsrq";
                emailTemplate = EmailTemplateEnum.INTERVIEW_THIRD.getValue();
            } else {
                return SUCCESS;
            }
            mainDataMap.put(msgField, msg);
            // 更新表单字段
            rs.executeUpdate("update " + billTableName + " set " + msgField + " = ? where id = ?", msg, billId);


            List<Formfield> fieldList = FormInfoManager.getInstance().getAllField(formId);
            Map<String, Formfield> fieldMap = fieldList.stream().collect(Collectors.toMap(Formfield::getFieldname, item -> item, (k1, k2) -> k1));


            //userIdSet.add(Util.null2String(mainDataMap.get(msgField)));
            String interviewDate = Util.null2String(mainDataMap.get(msrqField));
            String msgContent = RecruitCommon.getSettingValue(msgKey, fieldMap, mainDataMap);
            String msgCenterId = RecruitCommon.getSettingValue("MSG_CENTER_ID");
            String planTitleInterview = RecruitCommon.getSettingValue("PLAN_TITLE_INTERVIEW", fieldMap, mainDataMap);
            String planContentInterview = RecruitCommon.getSettingValue("PLAN_CONTENT_INTERVIEW", fieldMap, mainDataMap);
            String msgTitleInterview = RecruitCommon.getSettingValue("MSG_TITLE_INTERVIEW", fieldMap, mainDataMap);


            // 消息不为空，发送消息
            RecruitUtil.messagePush(msgCenterId, msgTitleInterview, msgContent, userIdSet, user.getUID());


            // 创建日程
            Map<String, Object> maps = new HashMap<>(16);
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
            LocalDateTime dateTime = LocalDateTime.parse(interviewDate, formatter);
            LocalDateTime newDateTime = dateTime.plusHours(1);
            // 日程类型
            maps.put("workPlanType", "0");
            // 工作计划名称
            maps.put("planName", planTitleInterview);
            // 系统参与人
            maps.put("memberIDs", StringUtils.join(userIdSet, ","));
            // 紧急程度
            maps.put("urgentLevel", "1");
            // 开始时间
            maps.put("beginDateTime", interviewDate);
            // 结束时间
            maps.put("endDateTime", newDateTime.format(formatter));
            // 内容
            maps.put("description", planContentInterview);
            // 相关文档
            maps.put("docIDs", mainDataMap.get("zp"));
            maps.put("requestIDs", requestId);
            String workId = getPlanIdByRequestId(rs, requestId);
            maps.put("workid", workId);

            Map<String, Object> result = ServiceUtil.getService(WorkPlanBaseServiceImpl.class, user).addWorkPlan(maps);
            Integer workplanId = (Integer) result.get("workplanid");
            rs.executeUpdate("update " + billTableName + " set msrc = ? where id = ?", workplanId, billId);


            // 发送邮件

            Map<String, List<Formfield>> fieldMapList = fieldList.stream().collect(Collectors.groupingBy(Formfield::getFieldname));
            String emailTitle = "";
            String emailContent = "";
            rs.executeQuery("select yjzt ,yjnr  from uf_recruit_email where mblx =? ", emailTemplate);
            if (rs.next()) {
                emailTitle = rs.getString("yjzt");
                emailContent = rs.getString("yjnr");
            }
            emailContent = RecruitUtil.getReplaceContent(emailContent, fieldMapList, mainDataMap);
            RecruitUtil.sendEmail(Util.null2String(mainDataMap.get("dzyx")), emailTitle, emailContent);

            return SUCCESS;
        } catch (Exception e) {
            rs.writeLog("InterviewAction error: ", e);
            requestInfo.getRequestManager().setMessagecontent(e.getMessage());
            return FAILURE_AND_CONTINUE;
        }
    }


    /**
     * 获取关联该流程的日程ID
     *
     * @param rs
     * @param requestId
     * @return
     */
    private String getPlanIdByRequestId(RecordSet rs, int requestId) {
        if (requestId < 0) {
            return "";
        }
        rs.executeQuery("select id from WorkPlan where requestid = ? ", requestId);
        if (rs.next()) {
            return rs.getString("id");
        }

        return "";
    }


    public String getInterviewStep() {
        return interviewStep;
    }

    public void setInterviewStep(String interviewStep) {
        this.interviewStep = interviewStep;
    }
}
